#R --max-ppsize=500000
rm(list=ls(all=FALSE))
ls()

library(foreach)
library(doParallel)
library(beepr)




# Duration: 1.98473 hours (Work MBP)
################################################################################
workingDirectory = '~/Desktop/RF/V4/'
region = 'v4'

animal.clostridiales<-c("pig",	"cattle",	"sewage", "dog", "ruminant", "herbivore")
animal.bacteroidales<-c("pig",	"cattle",	"sewage", "dog", "ruminant", "herbivore")

## Random Forest
NbTree = 10000
Repeat = 100
################################################################################




### Preparation #################################################################
bacterialgroup<-c("clostridiales", "bacteroidales")
#build the metadata matrix (have to change if more bacterial groups)
metadata1<-matrix(data = NA, nrow = length(animal.clostridiales),  ncol = 4)
metadata1[,1]<-1; metadata1[,2]<-rep(1:length(animal.clostridiales), 1)
metadata2<-matrix(data = NA, nrow = length(animal.bacteroidales),  ncol = 4)
metadata2[,1]<-2; metadata2[,2]<-rep(1:length(animal.bacteroidales), 1)
metadata<-rbind(metadata1, metadata2)
# set the directory paths
Directory_RF<-paste0(workingDirectory, "RFclassification/")
Directory_training<-paste0(workingDirectory, "data/training/")
#setup parallel backend to use many processors
cores=detectCores()
cl <- makeCluster(cores[1]-1) #not to overload your computer
registerDoParallel(cl)
################################################################################




time.begin=Sys.time()
foreach(x=1:nrow(metadata)) %dopar%  {
  
  
  ### Preparation #################################################################
  library(randomForest)
  i=metadata[x,1]
  j=metadata[x,2]
  
  BacterialGroup<-bacterialgroup[i]
  
  if(BacterialGroup=="clostridiales"){# only works if Clostridiales and Bacteroidales
    animal<-animal.clostridiales
  } else {
    animal<-animal.bacteroidales
  }
  Animal=animal[j]
  
  DatasetName<-paste0(Directory_training, BacterialGroup, '_count.txt')
  TypeName<-paste0(Directory_training, "SampleType_", BacterialGroup, ".txt")
  OutputDirectory<-paste0(Directory_RF, BacterialGroup, "_RF_MDGcomputing_", Repeat, "x_", NbTree, "trees/"); dir.create(OutputDirectory)
  OutputFilename<-paste0(OutputDirectory, "RF_MDG_", BacterialGroup, "_", Animal, ".txt")
  
  
  
  
  ### Import data ################################################################
  training.count<-read.table(DatasetName, header=TRUE, sep="\t", row.names = 1)
  sample.type<-read.table(TypeName, header=TRUE, sep="\t")
  type<-as.vector(sample.type[,c(Animal)])
  MDG<-matrix(data = NA, nrow = ncol(training.count),  ncol = Repeat); row.names(MDG)<-colnames(training.count)
  

  ### Relative abundance calculation #############################################
  training.percent<-sweep(training.count, 1, rowSums(training.count), '/')
  
  
  
  ### Merge type and data and delete extra samples ###############################
  data.training<-cbind(type, training.percent)
  data.training.subset<-subset(data.training, type!="delete")
  data.training.subset$type <- factor(data.training.subset$type) # remove "ghost" levels to avoid error message during the RF analysis
  
  ### Random Forest analysis ######################################################
  for (k in 1:Repeat) {
    RF.fit<-randomForest(as.factor(data.training.subset$type) ~ ., 
                         data = data.training.subset,
                         ntree = NbTree,
                         nodesize = 1,
                         importance = TRUE)
    MDG[,k] <- RF.fit$importance[,4]
  }
  
  write.table(MDG, OutputFilename, quote = FALSE, row.names = TRUE, col.names = TRUE, sep="\t")
}

#stop cluster
stopCluster(cl)


time.end<-Sys.time()
(duration<-time.end-time.begin)
beep(sound = 2, expr = NULL)

