rm(list=ls(all=FALSE))
ls()

library(foreach)
library(doParallel)
library(openxlsx)
library(beepr)#; beep(sound = 2, expr = NULL)



################################################################################
workingDirectory = '~/Desktop/RF/V4/'
region = 'v4'

animal.clostridiales<-c("pig",	"cattle",	"sewage", "dog", "ruminant", "herbivore")
animal.bacteroidales<-c("pig",	"cattle",	"sewage", "dog", "ruminant", "herbivore")

## Random Forest nº1
NbTree1 = 10000
Repeat1 = 100

## Random Forest nº2
NbTree2 = 1000
################################################################################





### Preparation #################################################################
bacterialgroup<-c("clostridiales", "bacteroidales")
#build the metadata matrix
metadata1<-matrix(data = NA, nrow = length(animal.clostridiales),  ncol = 4)
metadata1[,1]<-1; metadata1[,3]<-rep(1:length(animal.clostridiales), 1); metadata1[,4]<-animal.clostridiales
metadata2<-matrix(data = NA, nrow = length(animal.bacteroidales),  ncol = 4)
metadata2[,1]<-2; metadata2[,3]<-rep(1:length(animal.bacteroidales), 1); metadata2[,4]<-animal.bacteroidales
metadata<-rbind(metadata1, metadata2)
#metadata[,2]<-bacterialgroup[as.numeric(metadata[,1])]
#metadata[,4]<-animal
# Paths to directories/files
Directory_RF<-paste0(workingDirectory, "RFclassification/")
Directory_RF_training<-paste0(Directory_RF, "training/"); dir.create(Directory_RF_training)
Directory_RF_result<-paste0(Directory_RF, "results/"); dir.create(Directory_RF_result)
Directory_training<-paste0(workingDirectory, "data/training/")
OutputFileName_Info<-paste0(Directory_RF_result, "info.xlsx")
#setup parallel backend to use many processors
cores=detectCores()
cl <- makeCluster(cores[1]-1) #not to overload your computer
registerDoParallel(cl)
################################################################################






time.begin=Sys.time()
output=foreach(x=1:nrow(metadata), .combine = ) %dopar%  {
  
  
  ### Preparation #################################################################
  library(randomForest)
  
  i=as.numeric(metadata[x,1])
  j=as.numeric(metadata[x,3])
  
  BacterialGroup<-bacterialgroup[i]
  
  if(BacterialGroup=="clostridiales"){# only works if Clostridiales and Bacteroidales
    animal<-animal.clostridiales
  } else {
    animal<-animal.bacteroidales
  }
  Animal=animal[j]
  
  #BacterialGroup<-bacterialgroup[as.numeric(metadata[x,1])]
  #Animal=animal[as.numeric(metadata[x,3])]
  
  Filename_MDG<-paste0(Directory_RF, "MDG/", BacterialGroup, "_MDG_", Repeat1, "x_", NbTree1, "trees_", Animal, ".txt")
  OutputFilename_fit_RData<-paste0(Directory_RF_training, "fit_", BacterialGroup, "_", Animal, ".RData")
  OutputFilename_backup_RData<-paste0(Directory_RF_training, "RF_backup_", BacterialGroup, "_", Animal, ".RData")
  DatasetName<-paste0(Directory_training, BacterialGroup, '_count.txt')
  TypeName<-paste0(Directory_training, "SampleType_", BacterialGroup, ".txt")
  
  
  
  
  
  ### Import data ################################################################
  training.count<-read.table(DatasetName, header=TRUE, sep="\t", row.names = 1)
  sample.type<-read.table(TypeName, header=TRUE, sep="\t")
  type<-as.vector(sample.type[,c(Animal)])
  MDG.pre<-read.table(Filename_MDG, header=FALSE, sep="\t"); MDG<-as.vector(MDG.pre$V1)
  
  
  ### Select ASVs part of the classifiers - Create the custom training database ####
  training.count.select<-training.count[, colnames(training.count) %in% MDG]
  training.percent<-sweep(training.count.select, 1, rowSums(training.count.select), '/')
  training.percent.without0<-replace(training.percent, is.na(training.percent),0)
  data.training<-cbind(type, training.percent.without0)

  
  
  ### Training the machine learning ################################################
  for (k in 1:100) {
    RF.fit<-randomForest(as.factor(data.training$type) ~ ., 
                         data = data.training,
                         ntree = NbTree2,
                         nodesize = 1,
                         importance = TRUE)
    nam.rf <- paste("RF.fit.", k, sep = "")
    assign(nam.rf, RF.fit)
    
    error.1<-RF.fit.1$confusion[, !colnames(RF.fit.1$confusion) %in% "class.error"]
    error.2=error.1; diag(error.2)<-0
    oob.error<-(sum(error.2)/sum(error.1))*100
    nam.oob <- paste("oob.error.", k, sep = "")
    assign(nam.oob, oob.error)
  }
 
  
  fit<-combine(RF.fit.1,RF.fit.2,RF.fit.3,RF.fit.4,RF.fit.5,RF.fit.6,RF.fit.7,RF.fit.8,RF.fit.9,RF.fit.10,RF.fit.11,RF.fit.12,RF.fit.13,RF.fit.14,RF.fit.15,RF.fit.16,RF.fit.17,RF.fit.18,RF.fit.19,RF.fit.20,RF.fit.21,RF.fit.22,RF.fit.23,RF.fit.24,RF.fit.25,RF.fit.26,RF.fit.27,RF.fit.28,RF.fit.29,RF.fit.30,RF.fit.31,RF.fit.32,RF.fit.33,RF.fit.34,RF.fit.35,RF.fit.36,RF.fit.37,RF.fit.38,RF.fit.39,RF.fit.40,RF.fit.41,RF.fit.42,RF.fit.43,RF.fit.44,RF.fit.45,RF.fit.46,RF.fit.47,RF.fit.48,RF.fit.49,RF.fit.50,RF.fit.51,RF.fit.52,RF.fit.53,RF.fit.54,RF.fit.55,RF.fit.56,RF.fit.57,RF.fit.58,RF.fit.59,RF.fit.60,RF.fit.61,RF.fit.62,RF.fit.63,RF.fit.64,RF.fit.65,RF.fit.66,RF.fit.67,RF.fit.68,RF.fit.69,RF.fit.70,RF.fit.71,RF.fit.72,RF.fit.73,RF.fit.74,RF.fit.75,RF.fit.76,RF.fit.77,RF.fit.78,RF.fit.79,RF.fit.80,RF.fit.81,RF.fit.82,RF.fit.83,RF.fit.84,RF.fit.85,RF.fit.86,RF.fit.87,RF.fit.88,RF.fit.89,RF.fit.90,RF.fit.91,RF.fit.92,RF.fit.93,RF.fit.94,RF.fit.95,RF.fit.96,RF.fit.97,RF.fit.98,RF.fit.99,RF.fit.100)
  save(fit, file = OutputFilename_fit_RData)
  save(list = ls(pattern = "RF.fit."), file = OutputFilename_backup_RData)
  
  
  ### Output results ##############################################################
  oob.mean<-mean(oob.error.1, oob.error.2, oob.error.3, oob.error.4, oob.error.5, oob.error.6, oob.error.7, oob.error.8, oob.error.9, oob.error.10, oob.error.11, oob.error.12, oob.error.13, oob.error.14, oob.error.15, oob.error.16, oob.error.17, oob.error.18, oob.error.19, oob.error.20, oob.error.21, oob.error.22, oob.error.23, oob.error.24, oob.error.25, oob.error.26, oob.error.27, oob.error.28, oob.error.29, oob.error.30, oob.error.31, oob.error.32, oob.error.33, oob.error.34, oob.error.35, oob.error.36, oob.error.37, oob.error.38, oob.error.39, oob.error.40, oob.error.41, oob.error.42, oob.error.43, oob.error.44, oob.error.45, oob.error.46, oob.error.47, oob.error.48, oob.error.49, oob.error.50, oob.error.51, oob.error.52, oob.error.53, oob.error.54, oob.error.55, oob.error.56, oob.error.57, oob.error.58, oob.error.59, oob.error.60, oob.error.61, oob.error.62, oob.error.63, oob.error.64, oob.error.65, oob.error.66, oob.error.67, oob.error.68, oob.error.69, oob.error.70, oob.error.71, oob.error.72, oob.error.73, oob.error.74, oob.error.75, oob.error.76, oob.error.77, oob.error.78, oob.error.79, oob.error.80, oob.error.81, oob.error.82, oob.error.83, oob.error.84, oob.error.85, oob.error.86, oob.error.87, oob.error.88, oob.error.89, oob.error.90, oob.error.91, oob.error.92, oob.error.93, oob.error.94, oob.error.95, oob.error.96, oob.error.97, oob.error.98, oob.error.99, oob.error.100)
  list(BacterialGroup, Animal, length(MDG), oob.mean)
  
}

ouput.df<-as.data.frame(t(matrix(unlist(output), nrow=length(unlist(output[1])))))
names(ouput.df)<-c("bacterialgroup", "source", "noASVs", "oob")
write.xlsx(ouput.df, OutputFileName_Info)

time.end<-Sys.time()
(duration<-time.end-time.begin)
beep(sound = 2, expr = NULL)


