rm(list=ls(all=FALSE))
ls()

library(randomForest)
library(openxlsx)
library(tidyr)
library(beepr)#; beep(sound = 2, expr = NULL)

# function - opposite of %in%
'%!in%' <- function(x,y)!('%in%'(x,y)) 




################################################################################
workingDirectory = '~/Desktop/RF/V4/'
region = 'v4'

animal.clostridiales<-c("pig",	"cattle",	"sewage", "dog", "ruminant", "herbivore")
animal.bacteroidales<-c("pig",	"cattle",	"sewage", "dog", "ruminant", "herbivore")


UserDataset = "test" #no file extension
 
## Random Forest nº1
NbTree1 = 10000
Repeat1 = 100

## Random Forest nº2
NbTree2 = 1000
################################################################################







### Preparation #################################################################
bacterialgroup<-c("clostridiales", "bacteroidales") 
Directory_RF_training<-paste0(workingDirectory, "RFclassification/training/")
Directory_RF_result<-paste0(workingDirectory, "RFclassification/results/")
Directory_user<-paste0(workingDirectory, "data/testing/")
User.Filename<-paste0(Directory_user, UserDataset, ".txt")
#build the metadata matrix
metadata1<-matrix(data = NA, nrow = length(animal.clostridiales),  ncol = 4)
metadata1[,1]<-1; metadata1[,2]<-"clostridiales"; metadata1[,3]<-rep(1:length(animal.clostridiales), 1); metadata1[,4]<-animal.clostridiales
metadata2<-matrix(data = NA, nrow = length(animal.bacteroidales),  ncol = 4)
metadata2[,1]<-2; metadata2[,2]<-"bacteroidales"; metadata2[,3]<-rep(1:length(animal.bacteroidales), 1); metadata2[,4]<-animal.bacteroidales
metadata<-rbind(metadata1, metadata2)
################################################################################





### Import data ################################################################
user<-read.table(User.Filename, h=T, sep = "\t",  row.names = 1)
user.t<-t(user)


### Preparation - output Files ###################################################
# RF results - voting tree probabilities
OutputFile_Results<-matrix(NA, nrow=ncol(user)+2, ncol=(length(animal.clostridiales)+length(animal.bacteroidales)))
row.names(OutputFile_Results)[3:nrow(OutputFile_Results)]<-names(user)
OutputFile_Results[1,]<-metadata[,2]
OutputFile_Results[2,]<-metadata[,4]
# Simplified RF results
OutputFile_ResultsSimplified<-matrix(NA, nrow=ncol(user), ncol=length(bacterialgroup))
row.names(OutputFile_ResultsSimplified)<-names(user)
colnames(OutputFile_ResultsSimplified)<-bacterialgroup





time.begin=Sys.time()
for (i in 1:length(bacterialgroup)) {
  
  
  ### Preparation #################################################################
  BacterialGroup<-bacterialgroup[i]
  
  if(BacterialGroup=="clostridiales"){# only works if Clostridiales and Bacteroidales
    animal<-animal.clostridiales
  } else {
    animal<-animal.bacteroidales
  }
  
  #OutputFile_Results[1,(1+(i-1)*length(animal))]<-as.character(BacterialGroup)
  OutputFile_TEMP_ResultsSimplified<-matrix(NA, nrow=ncol(user), ncol=length(animal)) # RF results - temporary matrix for simplified results
  OutputFile_TEMP_ResultsSimplified2<-matrix(NA, nrow=ncol(user), ncol=length(animal)) # RF results - temporary matrix for simplified results
  
  

  
  
  for (x in 1:length(animal)) { 
    
    
    ### Preparation #################################################################
    Animal=animal[x]
    Filename_MDG<-paste0(workingDirectory, "RFclassification/MDG/", BacterialGroup, "_MDG_", Repeat1, "x_", NbTree1, "trees_", Animal, ".txt")
    Fit_Training_RData<-paste0(Directory_RF_training, "fit_", BacterialGroup, "_", Animal, ".RData")
    
    
    
    ### Import data ################################################################
    MDG.pre<-read.table(Filename_MDG, header=FALSE, sep="\t"); MDG<-as.vector(MDG.pre$V1)
    load(Fit_Training_RData)

    
    
    
    ##### Creation of the user dataset #############################################
    user.t.select<-user.t[, colnames(user.t) %in% MDG]
    ASVs.not.in.user.dataset<-MDG[which(MDG %!in% colnames(user.t.select))]
    user.t.ASVs.not.found<-matrix(data = 0, nrow = nrow(user.t),  ncol = length(ASVs.not.in.user.dataset))
    colnames(user.t.ASVs.not.found)<-ASVs.not.in.user.dataset; row.names(user.t.ASVs.not.found)<-row.names(user.t.select)
    user.t.all.ASVs<-cbind(user.t.select, user.t.ASVs.not.found)
    user.t.all.ASVs.ra<-sweep(user.t.all.ASVs, 1, rowSums(user.t.all.ASVs), '/')
    data.user<-replace(user.t.all.ASVs.ra, is.na(user.t.all.ASVs.ra),0)
    
    
    
    
    ##### Output results ##########################################################
    # prediction - probabilities
    prediction<-as.data.frame(predict(fit, data.user, "prob"))
    prediction.classifier<-round(prediction[, as.character(Animal)]*100, digits = 3)
    prediction.classifier[which(apply(data.user, 1, sum)==0)]<-0 #set prediction to zero if no ASV was found in the sample for a given classifier
    
    
    
    if(BacterialGroup=="clostridiales"){
      OutputFile_Results[3:nrow(OutputFile_Results), x]<-prediction.classifier
    } else {
      OutputFile_Results[3:nrow(OutputFile_Results), length(animal.clostridiales)+x]<-prediction.classifier
    }
    
    
    # prediction - simplified outputs
    prediction.classifier.df<-as.data.frame(prediction.classifier)
    OutputFile_TEMP_ResultsSimplified[,x]<-apply(prediction.classifier.df, 2, function(y) ifelse(y > 50, paste(Animal, "(a)", sep=""), ifelse(y > 45, paste(Animal, "(b)", sep=""), ifelse(y > 40, paste(Animal, "(c)", sep=""), paste("-")))))
    OutputFile_TEMP_ResultsSimplified2[,x]<-apply(prediction.classifier.df, 2, function(y) ifelse(y > 40, paste0(Animal), paste("-")))
    
  }
  
  OutputFile_TEMP_ResultsSimplified2<-as.data.frame(OutputFile_TEMP_ResultsSimplified2)
  names(OutputFile_TEMP_ResultsSimplified2)<-animal
  row.names(OutputFile_TEMP_ResultsSimplified2)<-row.names(data.user)
  write.table(OutputFile_TEMP_ResultsSimplified2,  paste0(Directory_RF_result, UserDataset, "_predictions_", BacterialGroup, ".txt"), quote = FALSE, row.names = TRUE, col.names = TRUE, sep = "\t")
  
  OutputFile_TEMP_ResultsSimplified.char<-as.character(apply(OutputFile_TEMP_ResultsSimplified, 1, paste, collapse="/"))
  OutputFile_ResultsSimplified[,i]<-gsub("/-", "", gsub("-/", "", OutputFile_TEMP_ResultsSimplified.char))

   
}
  


### Export outputs ###############################################################
#write.table(OutputFile_Results, paste0(Directory_RF_result, UserDataset, "_predictions_probabilities.txt"), sep="\t", quote=FALSE, col.names = FALSE, row.names = TRUE)
#write.table(OutputFile_ResultsSimplified, paste0(Directory_RF_result, UserDataset, "_predictions_simplified.txt"), sep="\t", quote=FALSE, col.names = TRUE, row.names = TRUE)

Workbook <- createWorkbook("Predictions") 
addWorksheet(Workbook, sheetName = "probabilities") 
writeData(Workbook, sheet = "probabilities", x = OutputFile_Results, colNames = F, rowNames = T, keepNA = FALSE) 
addWorksheet(Workbook, sheetName = "simplified results") 
writeData(Workbook, sheet = "simplified results", x = OutputFile_ResultsSimplified, colNames = T, rowNames = T, withFilter=TRUE) 
saveWorkbook(Workbook, paste0(Directory_RF_result, UserDataset, "_predictions.xlsx"), overwrite = TRUE)




time.end<-Sys.time()
  (duration<-time.end-time.begin)
  beep(sound = 2, expr = NULL)
  
