#!/usr/bin/python

import os
import sys
import argparse
import subprocess
from Bio import SeqIO

count_table = {}
sequence_to_asv_name = {}
sample_names = set([])

def main(infile, outfile):
    for seq_record in SeqIO.parse("../training/ASVSelectedBasedMDG.fa", "fasta"):
        count_table[seq_record.seq] = {}
        sequence_to_asv_name[seq_record.seq] = seq_record.id

    for seq_record in SeqIO.parse(infile, "fasta"):
        sequence = seq_record.seq
        sample_name = seq_record.id.split('|')[0]
        sample_names.add(sample_name)

        if sequence in count_table:
            if not sample_name in count_table[sequence]:
                count_table[sequence][sample_name] = 0

            count_table[sequence][sample_name] += 1

    output = open(outfile, 'w')
    output.write("\t".join(sample_names) + "\n")

    for sequence in count_table:
        line = sequence_to_asv_name[sequence] 

        for sample_name in sample_names:
            line += '\t' + str(count_table[sequence].get(sample_name, 0))

        output.write(line + '\n')
    output.close()

main(sys.argv[1], sys.argv[2])
